//	DialogBoxCloseButton.cpp - LookAndFeel subclass so that dialogs have
//							   the same close button as the main window.
//	--------------------------------------------------------------------------
//	This file is part of 'Niall's Pedal Board', a simple ASIO VST host.
//	Copyright (C) 2004-2005  Niall Moody
//	
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//	--------------------------------------------------------------------------
//	
//	You can e-mail me at niallmoody@yahoo.co.uk if you want.
//	
//	--------------------------------------------------------------------------

#include "DialogBoxCloseButton.h"
//#include "NPBImages.h"
#include "FactoryObject.h" //(for the convertRelativeFilePath function)

DialogBoxCloseButton::DialogBoxCloseButton(String image1,
										   String image2,
										   String image3,
										   Colour resizerColour):
LookAndFeel(),
_image1(image1),
_image2(image2),
_image3(image3),
_resizerColour(resizerColour)
{
}

void DialogBoxCloseButton::drawButtonBackground (Graphics& g,
                                        Button& button,
                                        const Colour& backgroundColour,
                                        bool isMouseOverButton,
                                        bool isButtonDown)
{
    const int width = button.getWidth();
    const int height = button.getHeight();

    const float indent = 2.0f;
    const int cornerSize = jmin (roundFloatToInt (width * 0.4f),
                                 roundFloatToInt (height * 0.4f));

    Path p;
    p.addRoundedRectangle (indent, indent,
                           width - indent * 2.0f,
                           height - indent * 2.0f,
                           (float)cornerSize);

    Colour bc (backgroundColour);

    if (isMouseOverButton)
    {
        if (isButtonDown)
            bc = bc.brighter();
        else if (bc.getBrightness() > 0.5f)
            bc = bc.darker (0.1f);
        else
            bc = bc.brighter (0.1f);
    }

    g.setColour (bc);

    g.fillPath (p);

    //g.setColour (bc.contrasting().withAlpha ((isMouseOverButton) ? 0.5f : 0.4f));
	if(isMouseOverButton)
		g.setColour(bc.darker(0.4f));
	else
		g.setColour(bc.darker(0.3f));
    g.strokePath (p, PathStrokeType ((isMouseOverButton) ? 2.0f : 1.4f));
}

Button *DialogBoxCloseButton::createDialogBoxCloseButton()
{
	ImageButton *CloseButton;

	/*Image *CloseImage1 = ImageCache::getFromMemory(NPBImages::closebutton_1_png, NPBImages::closebutton_1_pngSize);
	Image *CloseImage2 = ImageCache::getFromMemory(NPBImages::closebutton_2_png, NPBImages::closebutton_2_pngSize);
	Image *CloseImage3 = ImageCache::getFromMemory(NPBImages::closebutton_3_png, NPBImages::closebutton_3_pngSize);*/

	_image1 = convertRelativeFilePath(_image1);
	_image2 = convertRelativeFilePath(_image2);
	_image3 = convertRelativeFilePath(_image3);

	Image *image1 = ImageCache::getFromFile(File(_image1));
	Image *image2 = ImageCache::getFromFile(File(_image2));
	Image *image3 = ImageCache::getFromFile(File(_image3));

	CloseButton = new ImageButton(T("Close"));
	CloseButton->setImages(true, true, true,
						   image1, 1.0f, Colours::transparentBlack,
						   image2, 1.0f, Colours::transparentBlack,
						   image3, 1.0f, Colours::transparentBlack,
						   0.5f);

	return CloseButton;
}

void DialogBoxCloseButton::drawCornerResizer(Graphics& g,
                                    int w, int h,
                                    bool isMouseOver,
                                    bool isMouseDragging)
{
	g.setColour (isMouseOver ? _resizerColour.darker(0.2f)
                             : _resizerColour);

    const float lineThickness = jmin (w, h) * 0.1f;

    for (float i = 0.0f; i < 1.0f; i += 0.3f)
    {
        g.drawLine (w * i,
                    h + 1.0f,
                    w + 1.0f,
                    h * i,
                    lineThickness);
    }
}

DBCButtonShiny::DBCButtonShiny(String image1,
									 String image2,
									 String image3,
									 Colour resizerColour):
ShinyLookAndFeel(),
_image1(image1),
_image2(image2),
_image3(image3),
_resizerColour(resizerColour)
{
}

/*void DBCButtonShiny::drawButtonBackground (Graphics& g,
                                        Button& button,
                                        const Colour& backgroundColour,
                                        bool isMouseOverButton,
                                        bool isButtonDown)
{
    const int width = button.getWidth();
    const int height = button.getHeight();

    const float indent = 2.0f;
    const int cornerSize = jmin (roundFloatToInt (width * 0.4f),
                                 roundFloatToInt (height * 0.4f));

    Path p;
    p.addRoundedRectangle (indent, indent,
                           width - indent * 2.0f,
                           height - indent * 2.0f,
                           (float)cornerSize);

    Colour bc (backgroundColour);

    if (isMouseOverButton)
    {
        if (isButtonDown)
            bc = bc.brighter();
        else if (bc.getBrightness() > 0.5f)
            bc = bc.darker (0.1f);
        else
            bc = bc.brighter (0.1f);
    }

    g.setColour (bc);

    g.fillPath (p);

    //g.setColour (bc.contrasting().withAlpha ((isMouseOverButton) ? 0.5f : 0.4f));
	if(isMouseOverButton)
		g.setColour(bc.darker(0.4f));
	else
		g.setColour(bc.darker(0.3f));
    g.strokePath (p, PathStrokeType ((isMouseOverButton) ? 2.0f : 1.4f));
}*/

Button *DBCButtonShiny::createDialogBoxCloseButton()
{
	ImageButton *CloseButton;

	/*Image *CloseImage1 = ImageCache::getFromMemory(NPBImages::closebutton_1_png, NPBImages::closebutton_1_pngSize);
	Image *CloseImage2 = ImageCache::getFromMemory(NPBImages::closebutton_2_png, NPBImages::closebutton_2_pngSize);
	Image *CloseImage3 = ImageCache::getFromMemory(NPBImages::closebutton_3_png, NPBImages::closebutton_3_pngSize);*/

	_image1 = convertRelativeFilePath(_image1);
	_image2 = convertRelativeFilePath(_image2);
	_image3 = convertRelativeFilePath(_image3);

	Image *image1 = ImageCache::getFromFile(File(_image1));
	Image *image2 = ImageCache::getFromFile(File(_image2));
	Image *image3 = ImageCache::getFromFile(File(_image3));

	CloseButton = new ImageButton(T("Close"));
	CloseButton->setImages(true, true, true,
						   image1, 1.0f, Colours::transparentBlack,
						   image2, 1.0f, Colours::transparentBlack,
						   image3, 1.0f, Colours::transparentBlack,
						   0.5f);

	return CloseButton;
}

void DBCButtonShiny::drawCornerResizer(Graphics& g,
                                    int w, int h,
                                    bool isMouseOver,
                                    bool isMouseDragging)
{
	g.setColour (isMouseOver ? _resizerColour.darker(0.2f)
                             : _resizerColour);

    const float lineThickness = jmin (w, h) * 0.1f;

    for (float i = 0.0f; i < 1.0f; i += 0.3f)
    {
        g.drawLine (w * i,
                    h + 1.0f,
                    w + 1.0f,
                    h * i,
                    lineThickness);
    }
}
