///	FactoryObject.cpp - Base class of all objects constructed from the factory
///						(i.e. any component you want the user to be able to
///						place, customise etc.).
///	--------------------------------------------------------------------------
///	This file is part of 'Niall's Pedal Board', a simple ASIO VST host.
///	Copyright (C) 2004-2005  Niall Moody
///	
///	This program is free software; you can redistribute it and/or modify
///	it under the terms of the GNU General Public License as published by
///	the Free Software Foundation; either version 2 of the License, or
///	(at your option) any later version.
///	
///	This program is distributed in the hope that it will be useful,
///	but WITHOUT ANY WARRANTY; without even the implied warranty of
///	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
///	GNU General Public License for more details.
///	
///	You should have received a copy of the GNU General Public License
///	along with this program; if not, write to the Free Software
///	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
///	
///	--------------------------------------------------------------------------

#include <juce.h>

#include "FactoryObject.h"

ComponentFactory *FactoryObject::factory = 0;

///---------------------------------------------------------------------------
FactoryObject::FactoryObject(ComponentFactory *factory,
							 Component **windowComponent)
{
	this->factory = factory;
	this->windowComponent = windowComponent;

	x = -1;
	y = -1;
	xRel = -1.0f;
	yRel = -1.0f;
	width = -1;
	height = -1;
	widthRel = -1.0f;
	heightRel = -1.0f;
	enabled = true;
}

///---------------------------------------------------------------------------
String convertRelativeFilePath(String path)
{
	int i;
	String tempstr;
	String tempdir; //have to do some tricks to get the application directory...

	//this is to handle absolute file paths
#ifdef WIN32
	if(path[1] == T(':'))
		return path;
#else
	if(path[0] == T('/'))
		return path;
#endif

	tempdir = File::getCurrentApplicationFile().getFullPathName();
	for(i=(tempdir.length()-1);i>=0;i--)
	{
		if(tempdir[i] == File::separator)
			break;
	}
	tempstr = tempdir.substring(0, (i+1));

	tempstr += path;
	return tempstr;
}
