//	Joystick.cpp - Class to handle DirectInput Joysticks (Windows only).
//				   This code's a bit messy, I'm afraid...
//	--------------------------------------------------------------------------
//	This file is part of 'Niall's Pedal Board', a simple ASIO VST host.
//	Copyright (C) 2004-2005  Niall Moody
//	
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//	--------------------------------------------------------------------------
//	
//	You can e-mail me at niallmoody@yahoo.co.uk if you want.
//	
//	--------------------------------------------------------------------------

#include "Joystick.h"

BOOL CALLBACK DIEnumDevCallback(LPCDIDEVICEINSTANCE pinst, LPVOID pv);

Joystick::Joystick()
{
	int i;

	HINSTANCE hinst;
	HRESULT hr;
	String error = "";

	DirectInput = 0;
	Parent = (Component *)this;
	Ed = 0;
	EnumerateOnly = true;

	FilterOn = false;
	JoyFilter = new ParameterFilter *[NumControls];
	for(i=0;i<NumControls;i++)
		JoyFilter[i] = new ParameterFilter(44100.0f);

	initState();

	hinst = (HINSTANCE)PlatformUtilities::getCurrentModuleInstanceHandle();

	//Acquire a DirectInput object.
	hr = DirectInputCreate(hinst, DIRECTINPUT_VERSION, &DirectInput, NULL);

	/*//Enumerate the available joysticks.
	hr = IDirectInput_EnumDevices(DirectInput, DIDEVTYPE_JOYSTICK, DIEnumDevCallback, this, DIEDFL_ATTACHEDONLY);

	if(DirectInputDevice)
	{
		startTimer(1);
	}*/
}

Joystick::Joystick(Component *parent, TextEditor *debug)
{
	int i;
	HINSTANCE hinst;
	HRESULT hr;
	String error = "";

	DirectInput = 0;
	Parent = parent;
	Ed = debug;
	EnumerateOnly = false;

	FilterOn = false;
	JoyFilter = new ParameterFilter *[NumControls];
	for(i=0;i<NumControls;i++)
		JoyFilter[i] = new ParameterFilter(44100.0f);

	initState();

	hinst = (HINSTANCE)PlatformUtilities::getCurrentModuleInstanceHandle();

	//Acquire a DirectInput object.
	hr = DirectInputCreate(hinst, DIRECTINPUT_VERSION, &DirectInput, NULL);
	if FAILED(hr)
	{
		if(Ed)
		{
			int tempi = (int)hinst;
			error << "DirectInputCreate() failed : HINSTANCE=" << tempi << "hr=" << hr << ".\n";
			Ed->insertTextAtCursor(error);
		}
	}
	else
	{
		if(Ed)
		{
			error << "DirectInputCreate() succeeded!\n";
			Ed->insertTextAtCursor(error);
		}
	}
	error = "";

	//Enumerate the available joysticks.
	hr = IDirectInput_EnumDevices(DirectInput, DIDEVTYPE_JOYSTICK, DIEnumDevCallback, this, DIEDFL_ATTACHEDONLY);
	if FAILED(hr)
	{
		if(Ed)
		{
			int tempi = (int)hinst;
			error << "IDirectInput_EnumDevices() failed.\n";
			Ed->insertTextAtCursor(error);
		}
	}
	else
	{
		if(Ed)
		{
			error << "IDirectInput_EnumDevices() succeeded!\n";
			Ed->insertTextAtCursor(error);
		}
	}

	if(DirectInputDevice.size() == 1)
	{
		startTimer(1);
		//startThread();
	}
}

Joystick::~Joystick()
{
	int i;

	String error = "\n";

	if(isTimerRunning())
		stopTimer();
	/*if(isThreadRunning())
		stopThread(250);*/

	/*if(DirectInputDevice)
	{
		DirectInputDevice->Unacquire();
		if(Ed)
		{
			error << "Unacquired the DirectInputDevice object.\n";
			Ed->insertTextAtCursor(error);
		}
		error = "";
		DirectInputDevice->Release();
		if(Ed)
		{
			error << "Released the DirectInputDevice object.\n";
			Ed->insertTextAtCursor(error);
		}
		DirectInputDevice = 0;
	}*/
	for(i=0;i<DirectInputDevice.size();i++)
	{
		DirectInputDevice[i]->Unacquire();
		if(Ed)
		{
			error << "Unacquired the DirectInputDevice object." << i << "\n";
			Ed->insertTextAtCursor(error);
		}
		error = "";
		DirectInputDevice[i]->Release();
		if(Ed)
		{
			error << "Released the DirectInputDevice object." << i << "\n";
			Ed->insertTextAtCursor(error);
		}
	}
	error = "";
	if(DirectInput)
	{
		DirectInput->Release();
		if(Ed)
		{
			error << "Released the DirectInput object.\n";
			Ed->insertTextAtCursor(error);
		}
		DirectInput = 0;
	}

	//delete JoyFilter;
	for(i=0;i<NumControls;i++)
		delete JoyFilter[i];
	delete [] JoyFilter;
}

/*void Joystick::run()
{
	while(!threadShouldExit())
	{
		timerCallback();
	}
}*/

StringArray Joystick::enumerateDevices()
{
	Logger::writeToLog(T("Joystick::enumerateDevices(): About to enumerate devices."));
	IDirectInput_EnumDevices(DirectInput, DIDEVTYPE_JOYSTICK, DIEnumDevCallback, this, DIEDFL_ATTACHEDONLY);
	Logger::writeToLog(T("Joystick::enumerateDevices(): Devices enumerated."));

	return JoystickNameList;
}

void Joystick::acquireJoystick(String name)
{
	int i;
	HRESULT hr;
	LPDIRECTINPUTDEVICE did;

	for(i=0;i<JoystickNameList.size();i++)
	{
		if(JoystickNameList[i] == name)
		{
			hr = DirectInput->CreateDevice(JoystickList[i]->guidInstance, &did, NULL);
			if FAILED(hr)
			{
				if(hr == DIERR_DEVICENOTREG)
				{
					AlertWindow error(T("Joystick Error"), T("Device not registered."), AlertWindow::WarningIcon);
					error.addButton(T("OK"), 0);
					error.runModalLoop();
				}
				else if(hr == DIERR_INVALIDPARAM)
				{
					AlertWindow error(T("Joystick Error"), T("Invalid parameter."), AlertWindow::WarningIcon);
					error.addButton(T("OK"), 0);
					error.runModalLoop();
				}
				else if(hr == DIERR_NOINTERFACE)
				{
					AlertWindow error(T("Joystick Error"), T("No interface."), AlertWindow::WarningIcon);
					error.addButton(T("OK"), 0);
					error.runModalLoop();
				}
				else if(hr == DIERR_NOTINITIALIZED)
				{
					AlertWindow error(T("Joystick Error"), T("Not initialised."), AlertWindow::WarningIcon);
					error.addButton(T("OK"), 0);
					error.runModalLoop();
				}
				else if(hr == DIERR_OUTOFMEMORY)
				{
					AlertWindow error(T("Joystick Error"), T("Out of memory."), AlertWindow::WarningIcon);
					error.addButton(T("OK"), 0);
					error.runModalLoop();
				}
				break;
			}
			DirectInputDevice.add(did);
			DirectInputDevice.getLast()->SetDataFormat(&c_dfDIJoystick2);
			DirectInputDevice.getLast()->SetCooperativeLevel(static_cast<HWND>(Parent->getWindowHandle()),
															 DISCL_BACKGROUND|DISCL_NONEXCLUSIVE);

			DIPROPRANGE properties;
			//x-axis range
			properties.diph.dwSize = sizeof(properties); 
			properties.diph.dwHeaderSize = sizeof(properties.diph); 
			properties.diph.dwObj = DIJOFS_X; 
			properties.diph.dwHow = DIPH_BYOFFSET; 
			properties.lMin = -32767; 
			properties.lMax = +32767;
			DirectInputDevice.getLast()->SetProperty(DIPROP_RANGE, &properties.diph);
			//y-axis range
			properties.diph.dwSize = sizeof(properties); 
			properties.diph.dwHeaderSize = sizeof(properties.diph); 
			properties.diph.dwObj = DIJOFS_Y; 
			properties.diph.dwHow = DIPH_BYOFFSET; 
			properties.lMin = -32767; //why are these upside down?
			properties.lMax = +32767;
			DirectInputDevice.getLast()->SetProperty(DIPROP_RANGE, &properties.diph);
			//z-axis range
			properties.diph.dwSize = sizeof(properties); 
			properties.diph.dwHeaderSize = sizeof(properties.diph); 
			properties.diph.dwObj = DIJOFS_Z; 
			properties.diph.dwHow = DIPH_BYOFFSET; 
			properties.lMax = +32767;
			DirectInputDevice.getLast()->SetProperty(DIPROP_RANGE, &properties.diph);

			DIPROPDWORD tempthing;
			DirectInputDevice.getLast()->GetProperty(DIPROP_GRANULARITY, &tempthing.diph);
			DWORD tempthing2 = tempthing.dwData;

			DirectInputDevice.getLast()->Acquire();

			if(!isTimerRunning())
			{
				startTimer(1);
			}
			/*if(!isThreadRunning())
			{
				startThread();
			}*/

			break;
		}
	}
}

void Joystick::addToJoystickList(String joyname, LPCDIDEVICEINSTANCE pinst)
{
	DIDEVICEINSTANCE *tempinst = new DIDEVICEINSTANCE;

	tempinst->dwDevType = pinst->dwDevType;
	tempinst->dwSize = pinst->dwSize;
	tempinst->guidFFDriver = pinst->guidFFDriver;
	tempinst->guidInstance = pinst->guidInstance;
	strcpy(tempinst->tszInstanceName, pinst->tszInstanceName);
	strcpy(tempinst->tszProductName, pinst->tszProductName);
	tempinst->wUsage = pinst->wUsage;
	tempinst->wUsagePage = pinst->wUsagePage;

	JoystickNameList.add(joyname);
	JoystickList.add(tempinst);
}

void Joystick::addText(String& text)
{
	String tempstr;

	if(Ed)
		Ed->insertTextAtCursor(text);
	else
	{
		tempstr = T("Joystick::addText(): ");
		tempstr += text;
		Logger::writeToLog(text);
	}
}

void Joystick::createJoystickDevice(LPCDIDEVICEINSTANCE pinst)
{
	HRESULT hr;
	LPDIRECTINPUTDEVICE did;
	String error = "";

	//Create the input device.
	hr = DirectInput->CreateDevice(pinst->guidInstance, &did, NULL);
	if FAILED(hr)
	{
		if(Ed)
		{
			error << "DirectInput->CreateDevice() failed! hr = " << hr << "\n";
			Ed->insertTextAtCursor(error);		
		}
		DirectInputDevice = 0;
	}
	else
	{
		DirectInputDevice.add(did);
		if(Ed)
		{
			error << "DirectInput->CreateDevice() succeeded!\n";
			Ed->insertTextAtCursor(error);		
		}
	}
	error = "";

	//Set the device's data format.
	hr = DirectInputDevice.getLast()->SetDataFormat(&c_dfDIJoystick2);
	if FAILED(hr)
	{
		if(Ed)
		{
			error << "DirectInputDevice->SetDataFormat() failed!\n";
			Ed->insertTextAtCursor(error);		
		}
	}
	else
	{
		if(Ed)
		{
			error << "DirectInputDevice->SetDataFormat() succeeded!\n";
			Ed->insertTextAtCursor(error);		
		}
	}
	error = "";

	//Set the device's cooperative level.
	hr = DirectInputDevice.getLast()->SetCooperativeLevel((HWND)Parent->getWindowHandle(), DISCL_BACKGROUND|DISCL_EXCLUSIVE);
	if FAILED(hr)
	{
		if(Ed)
		{
			error << "DirectInputDevice->SetCooperativeLevel() failed!\n";
			Ed->insertTextAtCursor(error);		
		}
	}
	else
	{
		if(Ed)
		{
			error << "DirectInputDevice->SetCooperativeLevel() succeeded!\n";
			Ed->insertTextAtCursor(error);		
		}
	}
	error = "";

	//Set the device's properties.
	DIPROPRANGE properties;
	//x-axis range
	properties.diph.dwSize = sizeof(properties); 
	properties.diph.dwHeaderSize = sizeof(properties.diph); 
	properties.diph.dwObj = DIJOFS_X; 
	properties.diph.dwHow = DIPH_BYOFFSET; 
	properties.lMin = -32767; 
	properties.lMax = +32767;
	hr = DirectInputDevice.getLast()->SetProperty(DIPROP_RANGE, &properties.diph);
	if FAILED(hr)
	{
		if(Ed)
		{
			error << "DirectInputDevice->SetProperty() (range x) failed!\n";
			Ed->insertTextAtCursor(error);		
		}
	}
	else
	{
		if(Ed)
		{
			error << "DirectInputDevice->SetProperty() (range x) succeeded!\n";
			Ed->insertTextAtCursor(error);		
		}
	}
	error = "";
	//y-axis range
	properties.diph.dwSize = sizeof(properties); 
	properties.diph.dwHeaderSize = sizeof(properties.diph); 
	properties.diph.dwObj = DIJOFS_Y; 
	properties.diph.dwHow = DIPH_BYOFFSET; 
	properties.lMin = -32767; //why are these upside down?
	properties.lMax = +32767;
	hr = DirectInputDevice.getLast()->SetProperty(DIPROP_RANGE, &properties.diph);
	if FAILED(hr)
	{
		if(Ed)
		{
			error << "DirectInputDevice->SetProperty() (range y) failed!\n";
			Ed->insertTextAtCursor(error);		
		}
	}
	else
	{
		if(Ed)
		{
			error << "DirectInputDevice->SetProperty() (range y) succeeded!\n";
			Ed->insertTextAtCursor(error);		
		}
	}
	error = "";
	//z-axis range
	properties.diph.dwSize = sizeof(properties); 
	properties.diph.dwHeaderSize = sizeof(properties.diph); 
	properties.diph.dwObj = DIJOFS_Z; 
	properties.diph.dwHow = DIPH_BYOFFSET; 
	properties.lMax = +32767;
	hr = DirectInputDevice.getLast()->SetProperty(DIPROP_RANGE, &properties.diph);
	if FAILED(hr)
	{
		if(Ed)
		{
			error << "DirectInputDevice->SetProperty() (range z) failed!\n";
			Ed->insertTextAtCursor(error);		
		}
	}
	else
	{
		if(Ed)
		{
			error << "DirectInputDevice->SetProperty() (range z) succeeded!\n";
			Ed->insertTextAtCursor(error);		
		}
	}
	error = "";

	DIPROPDWORD tempthing;
	DWORD tempthing2;
	tempthing.diph.dwSize = sizeof(DIPROPDWORD);
	tempthing.diph.dwHeaderSize = sizeof(DIPROPHEADER);
	tempthing.diph.dwHow = DIPH_BYOFFSET;
	tempthing.diph.dwObj = DIJOFS_X;
	hr = DirectInputDevice.getLast()->GetProperty(DIPROP_GRANULARITY, &tempthing.diph);
	if(hr == DI_OK)
		tempthing2 = tempthing.dwData;
	else if(hr == DIERR_INVALIDPARAM)
		tempthing2 = 0;
	else if(hr == DIERR_NOTINITIALIZED)
		tempthing2 = 0;
	else if(hr == DIERR_OBJECTNOTFOUND)
		tempthing2 = 0;
	else if(hr == DIERR_UNSUPPORTED)
		tempthing2 = 0;
	else
		tempthing2 = -1;
	if(Ed)
	{
		error << "Granularity X = " << (int)tempthing2 << "\n";
		Ed->insertTextAtCursor(error);		
	}
	error = "";

	tempthing.diph.dwObj = DIJOFS_Y;
	hr = DirectInputDevice.getLast()->GetProperty(DIPROP_GRANULARITY, &tempthing.diph);
	if(hr == DI_OK)
		tempthing2 = tempthing.dwData;
	else if(hr == DIERR_INVALIDPARAM)
		tempthing2 = 0;
	else if(hr == DIERR_NOTINITIALIZED)
		tempthing2 = 0;
	else if(hr == DIERR_OBJECTNOTFOUND)
		tempthing2 = 0;
	else if(hr == DIERR_UNSUPPORTED)
		tempthing2 = 0;
	else
		tempthing2 = -1;
	if(Ed)
	{
		error << "Granularity Y = " << (int)tempthing2 << "\n";
		Ed->insertTextAtCursor(error);		
	}
	error = "";

	tempthing.diph.dwObj = DIJOFS_Z;
	hr = DirectInputDevice.getLast()->GetProperty(DIPROP_GRANULARITY, &tempthing.diph);
	if(hr == DI_OK)
		tempthing2 = tempthing.dwData;
	else if(hr == DIERR_INVALIDPARAM)
		tempthing2 = 0;
	else if(hr == DIERR_NOTINITIALIZED)
		tempthing2 = 0;
	else if(hr == DIERR_OBJECTNOTFOUND)
		tempthing2 = 0;
	else if(hr == DIERR_UNSUPPORTED)
		tempthing2 = 0;
	else
		tempthing2 = -1;
	if(Ed)
	{
		error << "Granularity Z = " << (int)tempthing2 << "\n";
		Ed->insertTextAtCursor(error);		
	}
	error = "";
}

void Joystick::acquireJoystick()
{
	HRESULT hr;
	String error = "";

	if(DirectInputDevice.size() == 1)
	{
		hr = DirectInputDevice.getLast()->Acquire();
		if FAILED(hr)
		{
			if(Ed)
			{
				error << "DirectInputDevice->Acquire() failed!\n";
				Ed->insertTextAtCursor(error);		
			}
		}
		else
		{
			if(Ed)
			{
				error << "DirectInputDevice->Acquire() succeeded!\n";
				Ed->insertTextAtCursor(error);		
			}
		}
		error = "";
	}
}

void Joystick::joystickCallback(JoystickEvent& joystate, int index)
{
	String text = "";

	if(Ed)
	{
		switch(joystate.getType())
		{
			case evAxis:
				text.printf("Joystick %d : Axis %d = %d\n", index, joystate.getIndex(), joystate.getAxis(joystate.getIndex()));
				Ed->insertTextAtCursor(text);
				break;
			case evRotation:
				text.printf("Joystick %d : Rotation %d = %d\n", index, joystate.getIndex(), joystate.getRotation(joystate.getIndex()));
				Ed->insertTextAtCursor(text);
				break;
			case evSlider:
				text.printf("Joystick %d : Slider %d = %d\n", index, joystate.getIndex(), joystate.getSlider(joystate.getIndex()));
				Ed->insertTextAtCursor(text);
				break;
			case evPOV:
				text.printf("Joystick %d : POV %d = %d\n", index, joystate.getIndex(), joystate.getPOV(joystate.getIndex()));
				Ed->insertTextAtCursor(text);
				break;
			case evButton:
				text.printf("Joystick %d : Button %d = %d\n", index, joystate.getIndex(), joystate.getButton(joystate.getIndex()));
				Ed->insertTextAtCursor(text);
				break;
			case evVelocity:
				text.printf("Joystick %d : Velocity %d = %d\n", index, joystate.getIndex(), joystate.getVelocity(joystate.getIndex()));
				Ed->insertTextAtCursor(text);
				break;
			case evAngularVelocity:
				text.printf("Joystick %d : AngularVelocity %d = %d\n", index, joystate.getIndex(), joystate.getAngularVelocity(joystate.getIndex()));
				Ed->insertTextAtCursor(text);
				break;
			case evSliderVelocity:
				text.printf("Joystick %d : SliderVelocity %d = %d\n", index, joystate.getIndex(), joystate.getSliderVelocity(joystate.getIndex()));
				Ed->insertTextAtCursor(text);
				break;
			case evAcceleration:
				text.printf("Joystick %d : Acceleration %d = %d\n", index, joystate.getIndex(), joystate.getAcceleration(joystate.getIndex()));
				Ed->insertTextAtCursor(text);
				break;
			case evAngularAcceleration:
				text.printf("Joystick %d : AngularAcceleration %d = %d\n", index, joystate.getIndex(), joystate.getAngularAcceleration(joystate.getIndex()));
				Ed->insertTextAtCursor(text);
				break;
			case evSliderAcceleration:
				text.printf("Joystick %d : SliderAcceleration %d = %d\n", index, joystate.getIndex(), joystate.getSliderAcceleration(joystate.getIndex()));
				Ed->insertTextAtCursor(text);
				break;
			case evForce:
				text.printf("Joystick %d : Force %d = %d\n", index, joystate.getIndex(), joystate.getForce(joystate.getIndex()));
				Ed->insertTextAtCursor(text);
				break;
			case evTorque:
				text.printf("Joystick %d : Torque %d = %d\n", index, joystate.getIndex(), joystate.getTorque(joystate.getIndex()));
				Ed->insertTextAtCursor(text);
				break;
			case evSliderForce:
				text.printf("Joystick %d : SliderForce %d = %d\n", index, joystate.getIndex(), joystate.getSliderForce(joystate.getIndex()));
				Ed->insertTextAtCursor(text);
				break;
		}
	}
}

void Joystick::timerCallback()
{
	int i, j;
	HRESULT hr;
	DIJOYSTATE2 joystate;
	JoystickEvent event;
	String error = "\n";

	for(i=0;i<DirectInputDevice.size();i++) //i.e. for each joystick attached to the system...
	{
		hr = ((IDirectInputDevice2 *)DirectInputDevice[i])->Poll();

		hr = DirectInputDevice[i]->GetDeviceState(sizeof(DIJOYSTATE2), &joystate);
		if(hr != DI_OK)
		{
			if(hr == DIERR_INPUTLOST)
			{
				if(Ed)
				{
					error << "DirectInputDevice->GetDeviceState() failed : lost input.\n";
					Ed->insertTextAtCursor(error);		
				}
				//acquireJoystick(); //hmm...
			}
			else if(hr == DIERR_INVALIDPARAM)
			{
				if(Ed)
				{
					error << "DirectInputDevice->GetDeviceState() failed : invalid parameter.\n";
					Ed->insertTextAtCursor(error);		
				}
			}
			else if(hr == DIERR_NOTACQUIRED)
			{
				if(Ed)
				{
					error << "DirectInputDevice->GetDeviceState() failed : not acquired.\n";
					Ed->insertTextAtCursor(error);		
				}
			}
			else if(hr == DIERR_NOTINITIALIZED)
			{
				if(Ed)
				{
					error << "DirectInputDevice->GetDeviceState() failed : not initialised.\n";
					Ed->insertTextAtCursor(error);		
				}
			}
			else if(hr == E_PENDING)
			{
				if(Ed)
				{
					error << "DirectInputDevice->GetDeviceState() failed : pending.\n";
					Ed->insertTextAtCursor(error);		
				}
			}
			else
			{
				if(Ed)
				{
					error << "DirectInputDevice->GetDeviceState() failed : dunno why...\n";
					Ed->insertTextAtCursor(error);		
				}
			}
		}
		OwnedArray<JoystickEvent> events;
		setEvent(joystate, events);
		for(j=0;j<events.size();j++)
			joystickCallback(*events[j], i);
	}
}

void Joystick::setEvent(DIJOYSTATE2 state, OwnedArray<JoystickEvent>& events)
{
	int i;
	JoystickEvent *tempevent;

	for(i=0;i<4;i++)
	{
		if(state.rgdwPOV[i] != OldState.rgdwPOV[i])
		{
			OldState.rgdwPOV[i] = state.rgdwPOV[i];
			tempevent = new JoystickEvent;
			tempevent->create(evPOV, i, OldState.rgdwPOV[i]);
			events.add(tempevent);
			//break;
		}
	}

	for(i=0;i<128;i++)
	{
		if(state.rgbButtons[i] != OldState.rgbButtons[i])
		{
			OldState.rgbButtons[i] = state.rgbButtons[i];
			tempevent = new JoystickEvent;
			tempevent->create(evButton, i, OldState.rgbButtons[i]);
			events.add(tempevent);
			//break;
		}
	}

	if(!FilterOn)
	{
		if(state.lX != OldState.lX)
		{
			OldState.lX = state.lX;
			tempevent = new JoystickEvent;
			tempevent->create(evAxis, 0, OldState.lX);
			events.add(tempevent);
		}
		else if(state.lY != OldState.lY)
		{
			OldState.lY = state.lY;
			tempevent = new JoystickEvent;
			tempevent->create(evAxis, 1, OldState.lY);
			events.add(tempevent);
		}
		else if(state.lZ != OldState.lZ)
		{
			OldState.lZ = state.lZ;
			tempevent = new JoystickEvent;
			tempevent->create(evAxis, 2, OldState.lZ);
			events.add(tempevent);
		}
		else if(state.lRx != OldState.lRx)
		{
			OldState.lRx = state.lRx;
			tempevent = new JoystickEvent;
			tempevent->create(evRotation, 0, OldState.lRx);
			events.add(tempevent);
		}
		else if(state.lRy != OldState.lRy)
		{
			OldState.lRy = state.lRy;
			tempevent = new JoystickEvent;
			tempevent->create(evRotation, 1, OldState.lRy);
			events.add(tempevent);
		}
		else if(state.lRz != OldState.lRz)
		{
			OldState.lRz = state.lRz;
			tempevent = new JoystickEvent;
			tempevent->create(evRotation, 2, OldState.lRz);
			events.add(tempevent);
		}
		else if(state.rglSlider[0] != OldState.rglSlider[0])
		{
			OldState.rglSlider[0] = state.rglSlider[0];
			tempevent = new JoystickEvent;
			tempevent->create(evSlider, 0, OldState.rglSlider[0]);
			events.add(tempevent);
		}
		else if(state.rglSlider[1] != OldState.rglSlider[1])
		{
			OldState.rglSlider[1] = state.rglSlider[1];
			tempevent = new JoystickEvent;
			tempevent->create(evSlider, 1, OldState.rglSlider[1]);
			events.add(tempevent);
		}
		else if(state.lVX != OldState.lVX)
		{
			OldState.lVX = state.lVX;
			tempevent = new JoystickEvent;
			tempevent->create(evVelocity, 0, OldState.lVX);
			events.add(tempevent);
		}
		else if(state.lVY != OldState.lVY)
		{
			OldState.lVY = state.lVY;
			tempevent = new JoystickEvent;
			tempevent->create(evVelocity, 1, OldState.lVY);
			events.add(tempevent);
		}
		else if(state.lVZ != OldState.lVZ)
		{
			OldState.lVZ = state.lVZ;
			tempevent = new JoystickEvent;
			tempevent->create(evVelocity, 2, OldState.lVZ);
			events.add(tempevent);
		}
		else if(state.lVRx != OldState.lVRx)
		{
			OldState.lVRx = state.lVRx;
			tempevent = new JoystickEvent;
			tempevent->create(evAngularVelocity, 0, OldState.lVRx);
			events.add(tempevent);
		}
		else if(state.lVRy != OldState.lVRy)
		{
			OldState.lVRy = state.lVRy;
			tempevent = new JoystickEvent;
			tempevent->create(evAngularVelocity, 1, OldState.lVRy);
			events.add(tempevent);
		}
		else if(state.lVRz != OldState.lVRz)
		{
			OldState.lVRz = state.lVRz;
			tempevent = new JoystickEvent;
			tempevent->create(evAngularVelocity, 2, OldState.lVRz);
			events.add(tempevent);
		}
		else if(state.rglVSlider[0] != OldState.rglVSlider[0])
		{
			OldState.rglVSlider[0] = state.rglVSlider[0];
			tempevent = new JoystickEvent;
			tempevent->create(evSliderVelocity, 0, OldState.rglVSlider[0]);
			events.add(tempevent);
		}
		else if(state.rglVSlider[1] != OldState.rglVSlider[1])
		{
			OldState.rglVSlider[1] = state.rglVSlider[1];
			tempevent = new JoystickEvent;
			tempevent->create(evSliderVelocity, 1, OldState.rglVSlider[1]);
			events.add(tempevent);
		}
		else if(state.lAX != OldState.lAX)
		{
			OldState.lAX = state.lAX;
			tempevent = new JoystickEvent;
			tempevent->create(evAcceleration, 0, OldState.lAX);
			events.add(tempevent);
		}
		else if(state.lAY != OldState.lAY)
		{
			OldState.lAY = state.lAY;
			tempevent = new JoystickEvent;
			tempevent->create(evAcceleration, 1, OldState.lAY);
			events.add(tempevent);
		}
		else if(state.lAZ != OldState.lAZ)
		{
			OldState.lAZ = state.lAZ;
			tempevent = new JoystickEvent;
			tempevent->create(evAcceleration, 2, OldState.lAZ);
			events.add(tempevent);
		}
		else if(state.lARx != OldState.lARx)
		{
			OldState.lARx = state.lARx;
			tempevent = new JoystickEvent;
			tempevent->create(evAngularAcceleration, 0, OldState.lARx);
			events.add(tempevent);
		}
		else if(state.lARy != OldState.lARy)
		{
			OldState.lARy = state.lARy;
			tempevent = new JoystickEvent;
			tempevent->create(evAngularAcceleration, 1, OldState.lARy);
			events.add(tempevent);
		}
		else if(state.lARz != OldState.lARz)
		{
			OldState.lARz = state.lARz;
			tempevent = new JoystickEvent;
			tempevent->create(evAngularAcceleration, 2, OldState.lARz);
			events.add(tempevent);
		}
		else if(state.rglASlider[0] != OldState.rglASlider[0])
		{
			OldState.rglASlider[0] = state.rglASlider[0];
			tempevent = new JoystickEvent;
			tempevent->create(evSliderAcceleration, 0, OldState.rglASlider[0]);
			events.add(tempevent);
		}
		else if(state.rglASlider[1] != OldState.rglASlider[1])
		{
			OldState.rglASlider[1] = state.rglASlider[1];
			tempevent = new JoystickEvent;
			tempevent->create(evSliderAcceleration, 1, OldState.rglASlider[1]);
			events.add(tempevent);
		}
		else if(state.lFX != OldState.lFX)
		{
			OldState.lFX = state.lFX;
			tempevent = new JoystickEvent;
			tempevent->create(evForce, 0, OldState.lFX);
			events.add(tempevent);
		}
		else if(state.lFY != OldState.lAY)
		{
			OldState.lFY = state.lFY;
			tempevent = new JoystickEvent;
			tempevent->create(evForce, 1, OldState.lFY);
			events.add(tempevent);
		}
		else if(state.lFZ != OldState.lFZ)
		{
			OldState.lFZ = state.lFZ;
			tempevent = new JoystickEvent;
			tempevent->create(evForce, 2, OldState.lFZ);
			events.add(tempevent);
		}
		else if(state.lFRx != OldState.lFRx)
		{
			OldState.lFRx = state.lFRx;
			tempevent = new JoystickEvent;
			tempevent->create(evTorque, 0, OldState.lFRx);
			events.add(tempevent);
		}
		else if(state.lFRy != OldState.lFRy)
		{
			OldState.lFRy = state.lFRy;
			tempevent = new JoystickEvent;
			tempevent->create(evTorque, 1, OldState.lFRy);
			events.add(tempevent);
		}
		else if(state.lFRz != OldState.lFRz)
		{
			OldState.lFRz = state.lFRz;
			tempevent = new JoystickEvent;
			tempevent->create(evTorque, 2, OldState.lFRz);
			events.add(tempevent);
		}
		else if(state.rglFSlider[0] != OldState.rglFSlider[0])
		{
			OldState.rglFSlider[0] = state.rglFSlider[0];
			tempevent = new JoystickEvent;
			tempevent->create(evSliderVelocity, 0, OldState.rglFSlider[0]);
			events.add(tempevent);
		}
		else if(state.rglFSlider[1] != OldState.rglFSlider[1])
		{
			OldState.rglFSlider[1] = state.rglFSlider[1];
			tempevent = new JoystickEvent;
			tempevent->create(evSliderVelocity, 1, OldState.rglFSlider[1]);
			events.add(tempevent);
		}
	}
	else
	{
		if((state.lX != OldState.lX)||(FiltState.lX != OldState.lX))
		{
			if(state.lX != OldState.lX)
				OldState.lX = state.lX;
			tempevent = new JoystickEvent;
			FiltState.lX = (long)JoyFilter[AxisX]->GetSample((float)OldState.lX);
			tempevent->create(evAxis, 0, FiltState.lX);
			events.add(tempevent);
		}
		/*else */if((state.lY != OldState.lY)||(FiltState.lY != OldState.lY))
		{
			if(state.lY != OldState.lY)
				OldState.lY = state.lY;
			tempevent = new JoystickEvent;
			FiltState.lY = (long)JoyFilter[AxisY]->GetSample((float)OldState.lY);
			tempevent->create(evAxis, 1, FiltState.lY);
			events.add(tempevent);
		}
		/*else */if((state.lZ != OldState.lZ)||(FiltState.lZ != OldState.lZ))
		{
			if(state.lZ != OldState.lZ)
				OldState.lZ = state.lZ;
			tempevent = new JoystickEvent;
			FiltState.lZ = (long)JoyFilter[AxisZ]->GetSample((float)OldState.lZ);
			tempevent->create(evAxis, 2, FiltState.lZ);
			events.add(tempevent);
		}
		/*else */if((state.lRx != OldState.lRx)||(FiltState.lRx != OldState.lRx))
		{
			if(state.lRx != OldState.lRx)
				OldState.lRx = state.lRx;
			tempevent = new JoystickEvent;
			FiltState.lRx = (long)JoyFilter[RotationX]->GetSample((float)OldState.lRx);
			tempevent->create(evRotation, 0, FiltState.lRx);
			events.add(tempevent);
		}
		/*else */if((state.lRy != OldState.lRy)||(FiltState.lRy != OldState.lRy))
		{
			if(state.lRy != OldState.lRy)
				OldState.lRy = state.lRy;
			tempevent = new JoystickEvent;
			FiltState.lRy = (long)JoyFilter[RotationY]->GetSample((float)OldState.lRy);
			tempevent->create(evRotation, 1, FiltState.lRy);
			events.add(tempevent);
		}
		/*else */if((state.lRz != OldState.lRz)||(FiltState.lRz != OldState.lRz))
		{
			if(state.lRz != OldState.lRz)
				OldState.lRz = state.lRz;
			tempevent = new JoystickEvent;
			FiltState.lRz = (long)JoyFilter[RotationZ]->GetSample((float)OldState.lRz);
			tempevent->create(evRotation, 2, FiltState.lRz);
			events.add(tempevent);
		}
		/*else */if((state.rglSlider[0] != OldState.rglSlider[0])||(FiltState.rglSlider[0] != OldState.rglSlider[0]))
		{
			if(state.rglSlider[0] != OldState.rglSlider[0])
				OldState.rglSlider[0] = state.rglSlider[0];
			tempevent = new JoystickEvent;
			FiltState.rglSlider[0] = (long)JoyFilter[SliderU]->GetSample((float)OldState.rglSlider[0]);
			tempevent->create(evSlider, 0, FiltState.rglSlider[0]);
			events.add(tempevent);
		}
		/*else */if((state.rglSlider[1] != OldState.rglSlider[1])||(FiltState.rglSlider[1] != OldState.rglSlider[1]))
		{
			if(state.rglSlider[1] != OldState.rglSlider[1])
				OldState.rglSlider[1] = state.rglSlider[1];
			tempevent = new JoystickEvent;
			FiltState.rglSlider[1] = (long)JoyFilter[SliderV]->GetSample((float)OldState.rglSlider[1]);
			tempevent->create(evSlider, 1, FiltState.rglSlider[1]);
			events.add(tempevent);
		}
		/*else */if((state.lVX != OldState.lVX)||(FiltState.lVX != OldState.lVX))
		{
			if(state.lVX != OldState.lVX)
				OldState.lVX = state.lVX;
			tempevent = new JoystickEvent;
			FiltState.lVX = (long)JoyFilter[VelocityX]->GetSample((float)OldState.lVX);
			tempevent->create(evVelocity, 0, FiltState.lVX);
			events.add(tempevent);
		}
		/*else */if((state.lVY != OldState.lVY)||(FiltState.lVY != OldState.lVY))
		{
			if(state.lVY != OldState.lVY)
				OldState.lVY = state.lVY;
			tempevent = new JoystickEvent;
			FiltState.lVY = (long)JoyFilter[VelocityY]->GetSample((float)OldState.lVY);
			tempevent->create(evVelocity, 1, FiltState.lVY);
			events.add(tempevent);
		}
		/*else */if((state.lVZ != OldState.lVZ)||(FiltState.lVZ != OldState.lVZ))
		{
			if(state.lVZ != OldState.lVZ)
				OldState.lVZ = state.lVZ;
			tempevent = new JoystickEvent;
			FiltState.lVZ = (long)JoyFilter[VelocityZ]->GetSample((float)OldState.lVZ);
			tempevent->create(evVelocity, 2, FiltState.lVZ);
			events.add(tempevent);
		}
		/*else */if((state.lVRx != OldState.lVRx)||(FiltState.lVRx != OldState.lVRx))
		{
			if(state.lVRx != OldState.lVRx)
				OldState.lVRx = state.lVRx;
			tempevent = new JoystickEvent;
			FiltState.lVRx = (long)JoyFilter[AngularVelocityX]->GetSample((float)OldState.lVRx);
			tempevent->create(evAngularVelocity, 0, FiltState.lVRx);
			events.add(tempevent);
		}
		/*else */if((state.lVRy != OldState.lVRy)||(FiltState.lVRy != OldState.lVRy))
		{
			if(state.lVRy != OldState.lVRy)
				OldState.lVRy = state.lVRy;
			tempevent = new JoystickEvent;
			FiltState.lVRy = (long)JoyFilter[AngularVelocityY]->GetSample((float)OldState.lVRy);
			tempevent->create(evAngularVelocity, 1, FiltState.lVRy);
			events.add(tempevent);
		}
		/*else */if((state.lVRz != OldState.lVRz)||(FiltState.lVRz != OldState.lVRz))
		{
			if(state.lVRz != OldState.lVRz)
				OldState.lVRz = state.lVRz;
			tempevent = new JoystickEvent;
			FiltState.lVRz = (long)JoyFilter[AngularVelocityZ]->GetSample((float)OldState.lVRz);
			tempevent->create(evAngularVelocity, 2, FiltState.lVRz);
			events.add(tempevent);
		}
		/*else */if((state.rglVSlider[0] != OldState.rglVSlider[0])||(FiltState.rglVSlider[0] != OldState.rglVSlider[0]))
		{
			if(state.rglVSlider[0] != OldState.rglVSlider[0])
				OldState.rglVSlider[0] = state.rglVSlider[0];
			tempevent = new JoystickEvent;
			FiltState.rglVSlider[0] = (long)JoyFilter[SliderVelocityU]->GetSample((float)OldState.rglVSlider[0]);
			tempevent->create(evSliderVelocity, 0, FiltState.rglVSlider[0]);
			events.add(tempevent);
		}
		/*else */if((state.rglVSlider[1] != OldState.rglVSlider[1])||(FiltState.rglVSlider[1] != OldState.rglVSlider[1]))
		{
			if(state.rglVSlider[1] != OldState.rglVSlider[1])
				OldState.rglVSlider[1] = state.rglVSlider[1];
			tempevent = new JoystickEvent;
			FiltState.rglVSlider[1] = (long)JoyFilter[SliderVelocityV]->GetSample((float)OldState.rglVSlider[1]);
			tempevent->create(evSliderVelocity, 1, FiltState.rglVSlider[1]);
			events.add(tempevent);
		}
		/*else */if((state.lAX != OldState.lAX)||(FiltState.lAX != OldState.lAX))
		{
			if(state.lAX != OldState.lAX)
				OldState.lAX = state.lAX;
			tempevent = new JoystickEvent;
			FiltState.lAX = (long)JoyFilter[AccelerationX]->GetSample((float)OldState.lAX);
			tempevent->create(evAcceleration, 0, FiltState.lAX);
			events.add(tempevent);
		}
		/*else */if((state.lAY != OldState.lAY)||(FiltState.lAY != OldState.lAY))
		{
			if(state.lAY != OldState.lAY)
				OldState.lAY = state.lAY;
			tempevent = new JoystickEvent;
			FiltState.lAY = (long)JoyFilter[AccelerationY]->GetSample((float)OldState.lAY);
			tempevent->create(evAcceleration, 1, FiltState.lAY);
			events.add(tempevent);
		}
		/*else */if((state.lAZ != OldState.lAZ)||(FiltState.lAZ != OldState.lAZ))
		{
			if(state.lAZ != OldState.lAZ)
				OldState.lAZ = state.lAZ;
			tempevent = new JoystickEvent;
			FiltState.lAZ = (long)JoyFilter[AccelerationZ]->GetSample((float)OldState.lAZ);
			tempevent->create(evAcceleration, 2, FiltState.lAZ);
			events.add(tempevent);
		}
		/*else */if((state.lARx != OldState.lARx)||(FiltState.lARx != OldState.lARx))
		{
			if(state.lARx != OldState.lARx)
				OldState.lARx = state.lARx;
			tempevent = new JoystickEvent;
			FiltState.lARx = (long)JoyFilter[AngularAccelerationX]->GetSample((float)OldState.lARx);
			tempevent->create(evAngularAcceleration, 0, FiltState.lARx);
			events.add(tempevent);
		}
		/*else */if((state.lARy != OldState.lARy)||(FiltState.lARy != OldState.lARy))
		{
			if(state.lARy != OldState.lARy)
				OldState.lARy = state.lARy;
			tempevent = new JoystickEvent;
			FiltState.lARy = (long)JoyFilter[AngularAccelerationY]->GetSample((float)OldState.lARy);
			tempevent->create(evAngularAcceleration, 1, FiltState.lARy);
			events.add(tempevent);
		}
		/*else */if((state.lARz != OldState.lARz)||(FiltState.lARz != OldState.lARz))
		{
			if(state.lARz != OldState.lARz)
				OldState.lARz = state.lARz;
			tempevent = new JoystickEvent;
			FiltState.lARz = (long)JoyFilter[AngularAccelerationZ]->GetSample((float)OldState.lARz);
			tempevent->create(evAngularAcceleration, 2, FiltState.lARz);
			events.add(tempevent);
		}
		/*else */if((state.rglASlider[0] != OldState.rglASlider[0])||(FiltState.rglASlider[0] != OldState.rglASlider[0]))
		{
			if(state.rglASlider[0] != OldState.rglASlider[0])
				OldState.rglASlider[0] = state.rglASlider[0];
			tempevent = new JoystickEvent;
			FiltState.rglASlider[0] = (long)JoyFilter[SliderAccelerationU]->GetSample((float)OldState.rglASlider[0]);
			tempevent->create(evSliderAcceleration, 0, FiltState.rglASlider[0]);
			events.add(tempevent);
		}
		/*else */if((state.rglASlider[1] != OldState.rglASlider[1])||(FiltState.rglASlider[1] != OldState.rglASlider[1]))
		{
			if(state.rglASlider[1] != OldState.rglASlider[1])
				OldState.rglASlider[1] = state.rglASlider[1];
			tempevent = new JoystickEvent;
			FiltState.rglASlider[1] = (long)JoyFilter[SliderAccelerationV]->GetSample((float)OldState.rglASlider[1]);
			tempevent->create(evSliderAcceleration, 1, FiltState.rglASlider[1]);
			events.add(tempevent);
		}
		/*else */if((state.lFX != OldState.lFX)||(FiltState.lFX != OldState.lFX))
		{
			if(state.lFX != OldState.lFX)
				OldState.lFX = state.lFX;
			tempevent = new JoystickEvent;
			FiltState.lFX = (long)JoyFilter[ForceX]->GetSample((float)OldState.lFX);
			tempevent->create(evForce, 0, FiltState.lFX);
			events.add(tempevent);
		}
		/*else */if((state.lFY != OldState.lFY)||(FiltState.lFY != OldState.lFY))
		{
			if(state.lFY != OldState.lFY)
				OldState.lFY = state.lFY;
			tempevent = new JoystickEvent;
			FiltState.lFY = (long)JoyFilter[ForceY]->GetSample((float)OldState.lFY);
			tempevent->create(evForce, 1, FiltState.lFY);
			events.add(tempevent);
		}
		/*else */if((state.lFZ != OldState.lFZ)||(FiltState.lFZ != OldState.lFZ))
		{
			if(state.lFZ != OldState.lFZ)
				OldState.lFZ = state.lFZ;
			tempevent = new JoystickEvent;
			FiltState.lFZ = (long)JoyFilter[ForceZ]->GetSample((float)OldState.lFZ);
			tempevent->create(evForce, 2, FiltState.lFZ);
			events.add(tempevent);
		}
		/*else */if((state.lFRx != OldState.lFRx)||(FiltState.lFRx != OldState.lFRx))
		{
			if(state.lFRx != OldState.lFRx)
				OldState.lFRx = state.lFRx;
			tempevent = new JoystickEvent;
			FiltState.lFRx = (long)JoyFilter[TorqueX]->GetSample((float)OldState.lFRx);
			tempevent->create(evTorque, 0, FiltState.lFRx);
			events.add(tempevent);
		}
		/*else */if((state.lFRy != OldState.lFRy)||(FiltState.lFRy != OldState.lFRy))
		{
			if(state.lFRy != OldState.lFRy)
				OldState.lFRy = state.lFRy;
			tempevent = new JoystickEvent;
			FiltState.lFRy = (long)JoyFilter[TorqueY]->GetSample((float)OldState.lFRy);
			tempevent->create(evTorque, 1, FiltState.lFRy);
			events.add(tempevent);
		}
		/*else */if((state.lFRz != OldState.lFRz)||(FiltState.lFRz != OldState.lFRz))
		{
			if(state.lFRz != OldState.lFRz)
				OldState.lFRz = state.lFRz;
			tempevent = new JoystickEvent;
			FiltState.lFRz = (long)JoyFilter[TorqueZ]->GetSample((float)OldState.lFRz);
			tempevent->create(evTorque, 2, FiltState.lFRz);
			events.add(tempevent);
		}
		/*else */if((state.rglFSlider[0] != OldState.rglFSlider[0])||(FiltState.rglFSlider[0] != OldState.rglFSlider[0]))
		{
			if(state.rglFSlider[0] != OldState.rglFSlider[0])
				OldState.rglFSlider[0] = state.rglFSlider[0];
			tempevent = new JoystickEvent;
			FiltState.rglFSlider[0] = (long)JoyFilter[SliderForceU]->GetSample((float)OldState.rglFSlider[0]);
			tempevent->create(evSliderForce, 0, FiltState.rglFSlider[0]);
			events.add(tempevent);
		}
		/*else */if((state.rglFSlider[1] != OldState.rglFSlider[1])||(FiltState.rglFSlider[1] != OldState.rglFSlider[1]))
		{
			if(state.rglFSlider[1] != OldState.rglFSlider[1])
				OldState.rglFSlider[1] = state.rglFSlider[1];
			tempevent = new JoystickEvent;
			FiltState.rglFSlider[1] = (long)JoyFilter[SliderForceV]->GetSample((float)OldState.rglFSlider[1]);
			tempevent->create(evSliderForce, 1, FiltState.rglFSlider[1]);
			events.add(tempevent);
		}
	}

	//return false;
}

void Joystick::initState()
{
	int i;

	OldState.lX = 0;
    OldState.lY = 0;
    OldState.lZ = 0;
    OldState.lRx = 0;
    OldState.lRy = 0;
    OldState.lRz = 0;

	for(i=0;i<2;i++)
		OldState.rglSlider[i] = 0;

	for(i=0;i<4;i++)
		OldState.rgdwPOV[i] = -1;

	for(i=0;i<128;i++)
		OldState.rgbButtons[i] = false;

    OldState.lVX = 0;
    OldState.lVY = 0;
    OldState.lVZ = 0;
    OldState.lVRx = 0;
    OldState.lVRy = 0;
    OldState.lVRz = 0;

	for(i=0;i<2;i++)
		OldState.rglVSlider[i] = 0;

    OldState.lAX = 0;
    OldState.lAY = 0;
    OldState.lAZ = 0;
    OldState.lARx = 0;
    OldState.lARy = 0;
    OldState.lARz = 0;

	for(i=0;i<2;i++)
		OldState.rglASlider[i] = 0;

    OldState.lFX = 0;
    OldState.lFY = 0;
    OldState.lFZ = 0;
    OldState.lFRx = 0;
    OldState.lFRy = 0;
    OldState.lFRz = 0;

	for(i=0;i<2;i++)
		OldState.rglFSlider[i] = 0;

	FiltState = OldState;

}

BOOL CALLBACK DIEnumDevCallback(LPCDIDEVICEINSTANCE pinst, LPVOID pv)
{
	String error = "";
	String error2 = "";
	Joystick *Joy = (Joystick *)pv;

	//guidInstance
	error = "\nguidInstance = ";
	error2.printf("%d", pinst->guidInstance.Data1);
	error += error2;
	error2.printf("%d", pinst->guidInstance.Data2);
	error += error2;
	error2.printf("%d", pinst->guidInstance.Data3);
	error += error2;
	error2.printf("%d", pinst->guidInstance.Data4);
	error += error2;
	error += "\n";
	Joy->addText(error);

	//guidProduct
	error = "guidProduct = ";
	error2.printf("%d", pinst->guidProduct.Data1);
	error += error2;
	error2.printf("%d", pinst->guidProduct.Data2);
	error += error2;
	error2.printf("%d", pinst->guidProduct.Data3);
	error += error2;
	error2.printf("%d", pinst->guidProduct.Data4);
	error += error2;
	error += "\n";
	Joy->addText(error);

	//dwDevType
	error = "dwDevType = ";
	switch(pinst->dwDevType&0x000000FF)
	{
		case DIDEVTYPE_MOUSE:
			error += "mouse";
			switch(pinst->dwDevType&0x0000FF00)
			{
				case DIDEVTYPEMOUSE_UNKNOWN:
					error += " : unknown";
					break;
				case DIDEVTYPEMOUSE_TRADITIONAL:
					error += " : traditional";
					break;
				case DIDEVTYPEMOUSE_FINGERSTICK:
					error += " : fingerstick?";
					break;
				case DIDEVTYPEMOUSE_TOUCHPAD:
					error += " : touch pad";
					break;
				case DIDEVTYPEMOUSE_TRACKBALL:
					error += " : trackpad";
					break;
			}
			break;
		case DIDEVTYPE_KEYBOARD:
			error += "keyboard";
			switch(pinst->dwDevType&0x0000FF00)
			{
				case DIDEVTYPEKEYBOARD_UNKNOWN:
					error += " : unknown";
					break;
				case DIDEVTYPEKEYBOARD_PCXT:
					error += " : pc xt";
					break;
				case DIDEVTYPEKEYBOARD_OLIVETTI:
					error += " : Olivetti";
					break;
				case DIDEVTYPEKEYBOARD_PCAT:
					error += " : pc at";
					break;
				case DIDEVTYPEKEYBOARD_PCENH:
					error += " : pc enchanced";
					break;
				case DIDEVTYPEKEYBOARD_NOKIA1050:
					error += " : nokia 1050";
					break;
				case DIDEVTYPEKEYBOARD_NOKIA9140:
					error += " : nokia 9140";
					break;
				case DIDEVTYPEKEYBOARD_NEC98:
					error += " : nec 98";
					break;
				case DIDEVTYPEKEYBOARD_NEC98LAPTOP:
					error += " : nec 98 (laptop)";
					break;
				case DIDEVTYPEKEYBOARD_NEC98106:
					error += " : nec 98 106-key";
					break;
				case DIDEVTYPEKEYBOARD_JAPAN106:
					error += " : japanese 106";
					break;
				case DIDEVTYPEKEYBOARD_JAPANAX:
					error += " : japanese ax";
					break;
				case DIDEVTYPEKEYBOARD_J3100:
					error += " : japanese j3100";
					break;
			}
			break;
		case DIDEVTYPE_JOYSTICK:
			error += "joystick";
			switch((pinst->dwDevType&0x0000FF00)>>8)
			{
				case DIDEVTYPEJOYSTICK_UNKNOWN:
					error += " : unknown";
					break;
				case DIDEVTYPEJOYSTICK_TRADITIONAL:
					error += " : traditional";
					break;
				case DIDEVTYPEJOYSTICK_FLIGHTSTICK:
					error += " : flight stick";
					break;
				case DIDEVTYPEJOYSTICK_GAMEPAD:
					error += " : gamepad";
					break;
				case DIDEVTYPEJOYSTICK_RUDDER:
					error += " : rudder";
					break;
				case DIDEVTYPEJOYSTICK_WHEEL:
					error += " : wheel";
					break;
				case DIDEVTYPEJOYSTICK_HEADTRACKER:
					error += " : headtracker";
					break;
				default:
					error += " : huh?";
					break;
			}
			break;
		case DIDEVTYPE_DEVICE:
			error += "device";
			break;
		default:
			error += "huh?";
			break;
	}
	if((pinst->dwDevType&0xFFFF0000) == DIDEVTYPE_HID)
		error += " : HID";
	else
		error += " : huh?";
	error += "\n";
	Joy->addText(error);

	//tszInstanceName
	error = "tszInstanceName = ";
	error += pinst->tszInstanceName;
	error += "\n";
	Joy->addText(error);

	//tszProductName
	error = "tszProductName = ";
	error += pinst->tszProductName;
	error += "\n";
	Joy->addText(error);

	//wUsagePage
	error = "wUsagePage = ";
	error2.printf("%d", pinst->wUsagePage);
	error += error2;
	error += "\n";
	Joy->addText(error);

	//wUsage
	error = "wUsage = ";
	error2.printf("%d", pinst->wUsage);
	error += error2;
	error += "\n\n";
	Joy->addText(error);

	if(!Joy->getEnumerateOnly())
	{
		Joy->createJoystickDevice(pinst);
		Joy->acquireJoystick();

		return DIENUM_STOP;
	}
	else
	{
		Joy->addToJoystickList(pinst->tszProductName, pinst);

		return DIENUM_CONTINUE;
	}
}

//----------------------------------------------------------------------
JoystickEvent::JoystickEvent(JoystickEventType type, int index, int val)
{
	int i;

	Type = type;
	Index = index;

	for(i=0;i<3;i++)
		Axis[i] = 0;
	for(i=0;i<3;i++)
		Rotation[i] = 0;
	for(i=0;i<2;i++)
		Slider[i] = 0;
	for(i=0;i<4;i++)
		POV[i] = 0;
	for(i=0;i<128;i++)
		Button[i] = false;
	for(i=0;i<3;i++)
		Velocity[i] = 0;
	for(i=0;i<3;i++)
		AngularVelocity[i] = 0;
	for(i=0;i<2;i++)
		SliderVelocity[i] = 0;
	for(i=0;i<3;i++)
		Acceleration[i] = 0;
	for(i=0;i<3;i++)
		AngularAcceleration[i] = 0;
	for(i=0;i<2;i++)
		SliderAcceleration[i] = 0;
	for(i=0;i<3;i++)
		Force[i] = 0;
	for(i=0;i<3;i++)
		Torque[i] = 0;
	for(i=0;i<2;i++)
		SliderForce[i] = 0;

	switch(Type)
	{
		case evAxis:
			Axis[index] = val;
			break;
		case evRotation:
			Rotation[index] = val;
			break;
		case evSlider:
			Slider[index] = val;
			break;
		case evPOV:
			POV[index] = val;
			break;
		case evButton:
			//Button[index] = (bool)val;
			Button[index] = val!=0;
			break;
		case evVelocity:
			Velocity[index] = val;
			break;
		case evAngularVelocity:
			AngularVelocity[index] = val;
			break;
		case evSliderVelocity:
			SliderVelocity[index] = val;
			break;
		case evAcceleration:
			Acceleration[index] = val;
			break;
		case evAngularAcceleration:
			AngularAcceleration[index] = val;
			break;
		case evSliderAcceleration:
			SliderAcceleration[index] = val;
			break;
		case evForce:
			Force[index] = val;
			break;
		case evTorque:
			Torque[index] = val;
			break;
		case evSliderForce:
			SliderForce[index] = val;
			break;
	}
}

void JoystickEvent::create(JoystickEventType type, int index, int val)
{
	int i;

	Type = type;
	Index = index;

	for(i=0;i<3;i++)
		Axis[i] = 0;
	for(i=0;i<3;i++)
		Rotation[i] = 0;
	for(i=0;i<2;i++)
		Slider[i] = 0;
	for(i=0;i<4;i++)
		POV[i] = 0;
	for(i=0;i<128;i++)
		Button[i] = false;
	for(i=0;i<3;i++)
		Velocity[i] = 0;
	for(i=0;i<3;i++)
		AngularVelocity[i] = 0;
	for(i=0;i<2;i++)
		SliderVelocity[i] = 0;
	for(i=0;i<3;i++)
		Acceleration[i] = 0;
	for(i=0;i<3;i++)
		AngularAcceleration[i] = 0;
	for(i=0;i<2;i++)
		SliderAcceleration[i] = 0;
	for(i=0;i<3;i++)
		Force[i] = 0;
	for(i=0;i<3;i++)
		Torque[i] = 0;
	for(i=0;i<2;i++)
		SliderForce[i] = 0;

	switch(Type)
	{
		case evAxis:
			Axis[index] = val;
			break;
		case evRotation:
			Rotation[index] = val;
			break;
		case evSlider:
			Slider[index] = val;
			break;
		case evPOV:
			POV[index] = val;
			break;
		case evButton:
			//Button[index] = (bool)val;
			Button[index] = val!=0;
			break;
		case evVelocity:
			Velocity[index] = val;
			break;
		case evAngularVelocity:
			AngularVelocity[index] = val;
			break;
		case evSliderVelocity:
			SliderVelocity[index] = val;
			break;
		case evAcceleration:
			Acceleration[index] = val;
			break;
		case evAngularAcceleration:
			AngularAcceleration[index] = val;
			break;
		case evSliderAcceleration:
			SliderAcceleration[index] = val;
			break;
		case evForce:
			Force[index] = val;
			break;
		case evTorque:
			Torque[index] = val;
			break;
		case evSliderForce:
			SliderForce[index] = val;
			break;
	}
}

int JoystickEvent::getVal()
{
	switch(Type)
	{
		case evAxis:
			return Axis[Index];
		case evRotation:
			return Rotation[Index];
		case evSlider:
			return Slider[Index];
		case evPOV:
			return POV[Index];
		case evButton:
			return Button[Index];
		case evVelocity:
			return Velocity[Index];
		case evAngularVelocity:
			return AngularVelocity[Index];
		case evSliderVelocity:
			return SliderVelocity[Index];
		case evAcceleration:
			return Acceleration[Index];
		case evAngularAcceleration:
			return AngularAcceleration[Index];
		case evSliderAcceleration:
			return SliderAcceleration[Index];
		case evForce:
			return Force[Index];
		case evTorque:
			return Torque[Index];
		case evSliderForce:
			return SliderForce[Index];
	}
	return 0;
}