//	ParameterFilter.cpp - A filter to smooth parameter changes, taken from music dsp.org:
//						  http://www.musicdsp.org/archive.php?classid=3#28
//	--------------------------------------------------------------------------
//	This file is part of 'Niall's Pedal Board', a simple ASIO VST host.
//	Copyright (C) 2004-2005  Niall Moody
//	
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//	--------------------------------------------------------------------------
//	
//	You can e-mail me at niallmoody@yahoo.co.uk if you want.
//	
//	--------------------------------------------------------------------------

#include "ParameterFilter.h"
#include <math.h>

ParameterFilter::ParameterFilter(float samplerate, double scale)
{
	Samplerate = samplerate;
	Scale = scale * Samplerate;
	//Smoothness = 0.999;
	Smoothness = 0.999;

	A = 1.0-(2.4/Scale);
	B = Smoothness;
	ACoeff = A;
	BCoeff = A*B;
	CCoeff = A*B*B;
	MasterGain = 1.0/(-1.0/(log(A)+2.0*log(B))+2.0/(log(A)+log(B))-1.0/log(A));
	AGain = MasterGain;
	BGain = MasterGain*(log(A*B*B)*(log(A)-log(A*B))/((log(A*B*B)-log(A*B))*log(A*B))-log(A)/log(A*B));
	CGain = MasterGain*(-(log(A)-log(A*B))/(log(A*B*B)-log(A*B)));

	AReg = 0.0;
	BReg = 0.0;
	CReg = 0.0;
}

float ParameterFilter::GetSample(float inval)
{
	double retval;

	AReg = ACoeff*AReg + (double)inval;
	BReg = BCoeff*BReg + (double)inval;
	CReg = CCoeff*CReg + (double)inval;

	retval = AGain*AReg + BGain*BReg + CGain*CReg;

	/*if(retval > 1.0)
		retval = 1.0;
	else if(retval < 0.0)
		retval = 0.0;*/

	return (float)retval;
}

void ParameterFilter::SetScale(double samples)
{
	Scale = samples * Samplerate;
	Smoothness = 0.999;

	A = 1.0-(2.4/Scale);
	B = Smoothness;
	ACoeff = A;
	BCoeff = A*B;
	CCoeff = A*B*B;
	MasterGain = 1.0/(-1.0/(log(A)+2.0*log(B))+2.0/(log(A)+log(B))-1.0/log(A));
	AGain = MasterGain;
	BGain = MasterGain*(log(A*B*B)*(log(A)-log(A*B))/((log(A*B*B)-log(A*B))*log(A*B))-log(A)/log(A*B));
	CGain = MasterGain*(-(log(A)-log(A*B))/(log(A*B*B)-log(A*B)));
}

void ParameterFilter::SetSmooth(double val)
{
	Smoothness = val;

	B = Smoothness;
	BCoeff = A*B;
	CCoeff = A*B*B;
	MasterGain = 1.0/(-1.0/(log(A)+2.0*log(B))+2.0/(log(A)+log(B))-1.0/log(A));
	AGain = MasterGain;
	BGain = MasterGain*(log(A*B*B)*(log(A)-log(A*B))/((log(A*B*B)-log(A*B))*log(A*B))-log(A)/log(A*B));
	CGain = MasterGain*(-(log(A)-log(A*B))/(log(A*B*B)-log(A*B)));
}

void ParameterFilter::SetSamplerate(float samplerate)
{
	Scale /= Samplerate;
	Samplerate = samplerate;
	Scale *= Samplerate;

	A = 1.0-(2.4/Scale);
	B = Smoothness;
	ACoeff = A;
	BCoeff = A*B;
	CCoeff = A*B*B;
	MasterGain = 1.0/(-1.0/(log(A)+2.0*log(B))+2.0/(log(A)+log(B))-1.0/log(A));
	AGain = MasterGain;
	BGain = MasterGain*(log(A*B*B)*(log(A)-log(A*B))/((log(A*B*B)-log(A*B))*log(A*B))-log(A)/log(A*B));
	CGain = MasterGain*(-(log(A)-log(A*B))/(log(A*B*B)-log(A*B)));
}